/*************************************************
*
*     PROJECT: RoundSLE
*  Programmer: J.A. Diego Kerejeta
*     Created: June 20, 2019
*
*     Masking for round authored by Jim Lee ( use of SetWindowRgn(), CreateRoundRectRgn() )
*
*     Version 0.1
*
**************************************************/

#include 'xbp.ch'
#include 'common.ch'
#include 'gra.ch'
#include 'dll.ch'
#include 'AppEvent.ch'

#define DEFAULT_RADIUS              25
#define DEFAULT_ICON_SIZE           16
#define DEFAULT_LEFT_MARGIN_ICON    10


*****************************
CLASS RoundSle FROM XbpStatic
*****************************
PROTECTED:
   VAR lEnter         // TRUE if the mouse is inside the object
   VAR oIcon          // The XbpIcon() object

   INLINE METHOD DrawIcon( oPS, aRect )
   ************************************
      LOCAL nYPos   := Int( ( aRect[ 4 ] - ::nIconSize ) / 2 )
      LOCAL aRectIco:= { ::nIconLeftMargin, nYPos, ::nIconLeftMargin + ::nIconSize - 1, nYPos + ::nIconSize - 1 }

      oPS:setColor( TranslateSysColor( IIF( ::controlState = XBP_STATE_DISABLED, XBPSYSCLR_DIALOGBACKGROUND, XBPSYSCLR_FIELDBACKGROUND ) ) )

      GraBox( oPS, aRectIco, { aRectIco[ 3 ], aRectIco[ 4 ] }, GRA_OUTLINEFILL )
      ::oIcon:draw( oPS, aRectIco, IIF( ::controlState == XBP_STATE_DISABLED, XBP_STATE_DISABLED, XBP_STATE_NORMAL ) )
   RETURN self

EXPORTED:
   VAR oSle              // The XbpSle() object
   VAR nIcon             // Id of the resource Icon to be displayed
   VAR nIconSize         // Icon size when draw
   VAR nIconLeftMargin   // Left margin in pixels separating the icon
   VAR nRadius           // How much to round the corners of the rectangle.

   INLINE METHOD destroy
   *********************
      ::setTrackPointer( .F. )
      ::XbpStatic:destroy()
      IF ::oIcon # nil ; ::oIcon:destroy() ; ::oIcon:= nil ; ENDIF
      ::lEnter:= ::oSle:= ::nIcon:= ::nIconSize:= ::nIconLeftMargin:= ::nRadius:= nil
   RETURN self

   INLINE METHOD Init( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *****************************************************************
      ::XbpStatic:init( oParent, oOwner, aPos, aSize, aPP, lVisible )
          ::clipChildren:= .T.
          ::drawMode:= XBP_DRAW_OWNER

      ::oSle:= XbpSle():new()

      ::lEnter         := .F.
      ::nRadius        := DEFAULT_RADIUS
      ::nIconSize      := DEFAULT_ICON_SIZE
      ::nIconLeftMargin:= DEFAULT_LEFT_MARGIN_ICON
      ::align          := ::oSle:align
      ::autoKeyboard   := ::oSle:autoKeyboard
      ::autoTab        := ::oSle:autoTab
      ::bufferLength   := ::oSle:bufferLength
      ::editable       := ::oSle:editable
      ::unReadable     := ::oSle:unReadable
      ::border         := .T.
   RETURN self

   INLINE METHOD Create( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *******************************************************************
      LOCAL nRgn, nCont, nSleXSize, nSleXPos
      DEFAULT lVisible TO ::visible, aPP TO {}

      AAdd( aPP, { XBP_PP_BGCLR, XBPSYSCLR_FIELDBACKGROUND } )

      ::autoSize:= .F.
      ::XbpStatic:create( oParent, oOwner, aPos, aSize, aPP, .F. )
      IF aSize = nil ; aSize:= ::currentSize() ; ENDIF
      IF ::nIcon # nil
         ::oIcon:= XbpIcon():new():create() ; ::oIcon:load( nil, ::nIcon, ::nIconSize, ::nIconSize )
         nSleXPos := ::nIconLeftMargin * 2 + ::nIconSize
         nSleXSize:= aSize[ 1 ] - ::nIconSize - ::nIconLeftMargin * 2 - 2
      ELSE
         nSleXPos := 7
         nSleXSize:= aSize[ 1 ] - 14
      ENDIF
      ::oSle:align       := ::align
      ::oSle:autoKeyboard:= ::autoKeyboard
      ::oSle:autoTab     := ::autoTab
      ::oSle:bufferLength:= ::bufferLength
      ::oSle:editable    := ::editable
      ::oSle:unReadable  := ::unReadable
      ::oSle:cuebanner   := ::cuebanner
      ::oSle:datalink    := ::datalink
      ::oSle:border      := .F.
      ::oSle:autoSize    := .F.
      ::oSle:create( self,, { nSleXPos, 4 },  { nSleXSize, aSize[ 2 ] - 8 } )
         ::oSle:setInputFocus := {|| ::invalidateRect() }
         ::oSle:killInputFocus:= {|| ::invalidateRect() }

      IF ! Empty( ::nRadius )
         FOR nCont:= 1 TO 50
            IF ( nRgn:= CreateRoundRectRgn(  0, aSize[ 2 ] + 1, aSize[ 1 ] + 1, 0, ::nRadius, ::nRadius ) ) > 0
               SetWindowRgn( ::getHWND(), nRgn, .T. ) ; DeleteObject( nRgn ) ; EXIT
            ENDIF
            Sleep( 1 )
         NEXT
      ENDIF
      ::setTrackPointer( .T. )
      IF lVisible = .T. ; ::XbpStatic:show() ; ENDIF
   RETURN self

   // runtime data

   VAR changed  
   VAR cueBanner 

   INLINE ACCESS ASSIGN METHOD Changed( lChanged ) VAR changed  ; RETURN ( ::changed:= ::oSle:changed( lChanged ) )

   // XbpSle configuration

   VAR align 
   VAR autoKeyboard 
   VAR autoSize
   VAR autoTab
   VAR border 
   VAR bufferLength  
   VAR editable
   VAR datalink
   VAR unReadable

   // XbpSle manipulation

   INLINE METHOD Clear                                  ; RETURN ::oSle:clear()
   INLINE METHOD CopyMarked                             ; RETURN ::oSle:copyMarked()
   INLINE METHOD CutMarked                              ; RETURN ::oSle:cutMarked()
   INLINE METHOD DeleteMarked                           ; RETURN ::oSle:deleteMarked()
   INLINE METHOD EditBuffer                             ; RETURN ::oSle:editBuffer()
   INLINE METHOD GetData                                ; RETURN ::oSle:getData()
   INLINE METHOD HideBalloonTip                         ; RETURN ::oSle:hideBalloonTip()
   INLINE METHOD PasteMarked                            ; RETURN ::oSle:pasteMarked()
   INLINE METHOD QueryFirstChar                         ; RETURN ::oSle:queryFirstChar()
   INLINE METHOD QueryMarked                            ; RETURN ::oSle:queryMarked()
   INLINE METHOD SetData( cBuffer )                     ; RETURN ::oSle:setData( cBuffer )
   INLINE METHOD SetFirstChar( nBufferPos )             ; RETURN ::oSle:setFirstChar( nBufferPos )
   INLINE METHOD SetMarked( aStartEnd )                 ; RETURN ::oSle:setMarked( aStartEnd )
   INLINE METHOD ShowBalloonTip( nType, cTitle, cText ) ; RETURN ::oSle:showBalloonTip( nType, cTitle, cText )

   // XbpSle settings

   INLINE METHOD SetEditable( lEditable )               ; RETURN ::oSle:setEditable( lEditable ) 
   INLINE METHOD SetInsertMode( lInsertMode )           ; RETURN ::oSle:setInsertMode( lInsertMode )

   // Generic

   INLINE METHOD Enter   ; ::lEnter:= .T.   ; ::invalidateRect() ; RETURN self
   INLINE METHOD Leave   ; ::lEnter:= .F.   ; ::invalidateRect() ; RETURN self
   INLINE METHOD disable ; ::oSle:disable()                      ; RETURN ::XbpStatic:disable()
   INLINE METHOD enable  ; ::oSle:enable()                       ; RETURN ::XbpStatic:enable()

   INLINE METHOD Resize( aOldSize, aNewSize )
   ******************************************
      ::oSle:setSize( { aNewSize[ 1 ] - IIF( ::nIcon = nil, 14, ::nIconSize + ::nIconLeftMargin * 2 + 2 ), aNewSize[ 2 ] - 8 } )
   RETURN self

   INLINE METHOD Draw( oPS, aInfo )
   ********************************
      IF ::border = .T.
         IF ::controlState = XBP_STATE_DISABLED
            oPS:setColor( TranslateSysColor( XBPSYSCLR_DIALOGBACKGROUND ) )
            GraBox( oPS, { 0, 0 }, { aInfo[ XBP_DRAWINFO_RECT, 3 ] - 1, aInfo[ XBP_DRAWINFO_RECT, 4 ] - 1 }, GRA_OUTLINEFILL, ::nRadius, ::nRadius )
            oPS:setColor( XBPSYSCLR_FIELDBACKGROUND )
            GraBox( oPS, { 1, 1 }, { aInfo[ XBP_DRAWINFO_RECT, 3 ] - 2, aInfo[ XBP_DRAWINFO_RECT, 4 ] - 2 }, GRA_OUTLINE, ::nRadius, ::nRadius )
            oPS:setColor( TranslateSysColor( XBPSYSCLR_SCROLLBAR ) )
         ELSEIF SetAppFocus() = ::oSle
            oPS:setColor( TranslateSysColor( XBPSYSCLR_HILITEBACKGROUND ) )
         ELSEIF ::lEnter = .T. .OR. ::oSle:controlState = XBP_STATE_HOT
            oPS:setColor( TranslateSysColor( XBPSYSCLR_BUTTONDEFAULT ) )
         ELSE
            oPS:setColor( TranslateSysColor( XBPSYSCLR_BUTTONDARK ) )
         ENDIF
         GraBox( oPS, { 0, 0 }, { aInfo[ XBP_DRAWINFO_RECT, 3 ] - 1, aInfo[ XBP_DRAWINFO_RECT, 4 ] - 1 }, GRA_OUTLINE, ::nRadius, ::nRadius )
     ELSEIF ::controlState = XBP_STATE_DISABLED
         oPS:setColor( TranslateSysColor( XBPSYSCLR_DIALOGBACKGROUND ) )
         GraBox( oPS, { 0, 0 }, { aInfo[ XBP_DRAWINFO_RECT, 3 ] - 1, aInfo[ XBP_DRAWINFO_RECT, 4 ] - 1 }, GRA_OUTLINEFILL, ::nRadius, ::nRadius )
     ENDIF   
     IF ::oIcon # nil ; ::drawIcon( oPS, aInfo[ XBP_DRAWINFO_RECT ] ) ; ENDIF
   RETURN .F.
ENDCLASS

DLLFUNCTION CreateRoundRectRgn( nX1, nY1, nX2, nY2, nW, nH ) USING STDCALL FROM GDI32.DLL
DLLFUNCTION SetWindowRgn( nHwnd, nRgn, lRedraw             ) USING STDCALL FROM USER32.DLL
DLLFUNCTION DeleteObject( nObject                          ) USING STDCALL FROM GDI32.DLL